@@ -35,4 +35,4 @@ coff:	subu $8, $31, $8
 ");
 
 #define _dl_boot _dl_boot2
-#define DL_BOOT(X)   static void __attribute__ ((unused)) _dl_boot (X)
+#define LD_BOOT(X)   static void __attribute__ ((unused)) _dl_boot (X)
